// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: string_index.h,v 1.7 2002/09/30 04:51:49 carzanig Exp $
//
#ifndef STRING_INDEX_H
#define STRING_INDEX_H

#include "fwd_table.h"
#include "tst.h"
#include "ft_allocator.h"

/** descriptor of a complete match in a <code>StringIndex</code> **/
class SICompleteDescr {
public:
    Constraint *		eq;
    Constraint *		lt;
    Constraint *		gt;
    Constraint *		sf;
    SICompleteDescr *		next_lt;
    SICompleteDescr *		next_gt;

    void			insert_between(TSTNode *, TSTNode *);
};

/** descriptor of a partial match in a <code>StringIndex</code> **/
struct SIPartialDescr {
    Constraint *		pf;
    Constraint *		ss;
};

/** index of string constraints.
 *
 *  This data structure supports <em>equals</em>, <em>less-than</em>,
 *  <em>greater-than</em>, <em>prefix</em>, <em>suffix</em>, and
 *  <em>substring</em> constraints on strings
 **/
class StringIndex : public PTST {
public:
				StringIndex();

    Constraint *		add_eq(const string & v, FTAllocator &);
    Constraint *		add_lt(const string & v, FTAllocator &);
    Constraint *		add_gt(const string & v, FTAllocator &);
    Constraint *		add_pf(const string & v, FTAllocator &);
    Constraint *		add_sf(const string & v, FTAllocator &);
    Constraint *		add_ss(const string & v, FTAllocator &);

    bool			match(const string & v, CProcessor & p) const;
};

#ifdef HAVE_INLINE
#include "string_index.icc"
#endif

#endif
